<?php

/**
 * VTB Payments callback
 *
 * @copyright  VTB
 * @author     VTB
 * @version    1.2.1
 *
 */

header('Content-type: text/html; charset=UTF-8');

require_once 'vendor/autoload.php';
if (!class_exists('Simpla')) {
    require_once '../../api/Simpla.php';
}

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException,
    \Vtbpay\Classes\Common\Outer;


class VtbPaymentsCallback extends \Simpla
{
    private $order; // Заказ

    private $orderId; // Идентификатор заказа

    private $paymentSettings; // Настройки

    private $paymentMethod; // Метод оплаты

    private $logger; // Logger


    /**
     * Constructs a new instance of VtbPaymentsCallback.
     */
    public function __construct()
    {
        $outer = $this->request->get('outer') ?: null;
        if ($outer === '1') {
            new Outer();
        }
        else {
            $this->init();
        }
    }


    /**
     * Initializes the callback.
     */
    private function init()
    {
        $this->setVtbpayLogger();
        $this->orderId = $this->request->get('orderId') ?: null;

        try {
            $this->checkOrder();

            $this->checkPaymentMethod();

            $this->setPaymentSettings();

            $this->checkOrderPaid();

            $this->checkGoods();

            $this->setAsPaid();

        } catch (\Exception | VtbPayException $e) {
            // Log the caught exception for debugging purposes.
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine()
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s; Order id: (%s)',
                $e->getMessage(),
                $this->orderId ?: ''
            ), $context);

            die($e->getMessage() . ' <a href="' . $this->config->root_url . '">Go Home</a>');
        }
    }


    /**
     * Checks the existence of an order.
     *
     * @throws \Exception Throws an exception if the order does not exist.
     */
    private function checkOrder()
    {
        if (empty($this->orderId)) {
            throw new \Exception('Идентификатор заказа не указан.');
        }

        // Существует ли заказ
        $this->order = $this->orders->get_order((int) $this->orderId);

        if (empty($this->order)) {
            throw new \Exception('Оплачиваемый заказ не найден.');
        }
    }


    /**
     * Checks the validity of the payment method.
     *
     * @throws \Exception Throws an exception if the payment method is invalid.
     */
    private function checkPaymentMethod()
    {
        // Наш ли способ оплаты
        $this->paymentMethod = $this->payment->get_payment_method(intval($this->order->payment_method_id));

        if (empty($this->paymentMethod)) {
            throw new \Exception('Неизвестный метод оплаты');
        }
    }


    /**
     * Sets the payment settings.
     *
     * @throws \Exception Throws an exception if the payment settings are missing.
     */
    private function setPaymentSettings()
    {
        $this->paymentSettings = $this->payment->get_payment_settings($this->paymentMethod->id);

        if (empty($this->paymentSettings)) {
            throw new \Exception('Настройки платёжной системы не указаны.');
        }
    }


    /**
     * Create and return an instance of the VtbApi class for VtbPayments integration.
     *
     * @return VtbApi An instance of the VtbApi class.
     */
    private function getVtbApi()
    {
        $settings = $this->paymentSettings;

        return new VtbApi(
            $settings['vtbpay_client_id'],
            $settings['vtbpay_client_secret'],
            (bool) $settings['vtbpay_test_mode'],
            $settings['vtbpay_merchant_authorization'] ?? ''
        );
    }


    /**
     * Checks if the order has been paid.
     *
     * @throws \Exception Throws an exception if the order has already been paid.
     */
    private function checkOrderPaid()
    {
        // Check if the order has already been paid
        if($this->order->paid) {
            throw new \Exception('Заказ уже оплачен.');
        }

        // Logging input
        $this->logger->debug(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
            'arguments' => [
                'orderId' => $this->orderId,
            ]
        ]);

        $response = $this->getVtbApi()->getOrderInfo($this->orderId);

        // Logging output
        $this->logger->debug(
            basename(__FILE__) . ' > ' . __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
            'response' => $response
        ]);

        // Если заказ оплачивался, и статус заказа уже изменялся
        if (
            !isset($response['object']['status']['value']) ||
            $response['object']['status']['value'] !== 'PAID'
        ) {
            throw new \Exception('Платёж не выполнен.');
        }
    }


    /**
     * Checks the availability of goods in the order.
     *
     * @throws \Exception Throws an exception if there is insufficient stock of any product.
     */
    private function checkGoods()
    {
        // Наличие товара
        $purchases = $this->orders->get_purchases([
            'order_id'=>intval($this->order->id)
        ]);

        foreach ($purchases as $purchase) {
            $variant = $this->variants->get_variant(intval($purchase->variant_id));

            if (empty($variant) || (!$variant->infinity && $variant->stock < $purchase->amount)) {
                throw new \Exception("Недостаточно товара {$purchase->product_name} {$purchase->variant_name}.");
            }
        }
    }


    /**
     * Marks the order as paid and performs necessary actions.
     */
    private function setAsPaid()
    {
        $this->orders->update_order($this->order->id, ['paid' => 1]);
        $this->orders->close($this->order->id);
        $this->orders->pay($this->order->id);
        $this->notify->email_order_user($this->order->id);
        $this->notify->email_order_admin($this->order->id);

        $success_url = $this->config->root_url . '/order/' . $this->order->url;
        header('location:' . $success_url, true, 302);
        exit;
    }


    /**
     * Устанавливает объект логирования для модуля.
     */
    public function setVtbpayLogger()
    {
        $logger = VtbPayLogger::getInstance();

        $this->logger = $logger
                        ->setOption('additionalCommonText', 'return-' . rand(1111, 9999))
                        ->setLogFilePath(
                            dirname(__DIR__, 2) . '/files/log/vtbpay-' . date('d-m-Y') . '.log'
                        )->setCustomRecording(function($message) use ($logger) {
                            if (
                                isset($this->paymentSettings['vtbpay_logging']) &&
                                $this->paymentSettings['vtbpay_logging']
                            ) $logger->writeToFile($message);
                        }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}

new VtbPaymentsCallback();
